/*
 * Tokenizer class
 * by hackervalley@free.fr
 * http://hackervalley.free.fr
 * May 2004
 * this program is licensed under the GPL.
 */

#ifndef TOKENIZER_H
#define TOKENIZER_H

#include <functional>
#include <string>
#include <vector>
#include <algorithm>

using namespace std;

class CSpace:public unary_function < char, bool >
{
      public:
	bool operator () (char c) const;
};

bool CSpace::operator()(char c) const
{
  return (' ' == c);
}

class CSeparator:public unary_function < char, bool >
{
      public:
	CSeparator::CSeparator (string const &rostr):m_ostr (rostr)
	{
	}
	bool operator () (char c) const;

      private:
	string m_ostr;
};

bool CSeparator::operator()(char c) const
{
  unsigned int iFind = m_ostr.find(c);
  if(iFind != string::npos)
    return true;
  else
    return false;
}

template < typename Pred = CSpace > class CTokenizer
{
      public:
	static void Tokenize (vector < string > &roResult,
			      string const &rostr, Pred const &roPred =
			      Pred ());
};

template < typename Pred >
	void CTokenizer < Pred >::Tokenize (vector < string > &roResult,
					    string const &rostr,
					    Pred const &roPred)
{
	/* Clear result vector */
	roResult.clear ();
	string::const_iterator it = rostr.begin ();
	string::const_iterator itTokenEnd = rostr.begin ();
	while (it != rostr.end ())
	{
		//Eat seperators
		while (roPred (*it))
			it++;
		//Find next token
		itTokenEnd = find_if (it, rostr.end (), roPred);
		//Append token to result
		if (it < itTokenEnd)
			roResult.push_back (string (it, itTokenEnd));
		it = itTokenEnd;
	}
}

#endif
